﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PackingEncoding.MultiComponentVector;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.MultiComponentVector {
  [Item("SingleGroupingMove", "A move on a multi component vector that is specified by a single index and a new grouping-value. The grouping of the item on the specified index is changed to the new value.")]
  [StorableClass]
  public class SingleGroupingMove : MultiComponentVectorMove {       
    [Storable]
    public int OldGroup { get; protected set; }
    [Storable]
    public int Index { get; protected set; }
    [Storable]
    public int NewGroup { get; protected set; }

    [StorableConstructor]
    protected SingleGroupingMove(bool deserializing) : base(deserializing) { }
    protected SingleGroupingMove(SingleGroupingMove original, Cloner cloner)
      : base(original, cloner) {
      this.OldGroup = original.OldGroup;
      this.Index = original.Index;
      this.NewGroup = original.NewGroup;
    }
    public SingleGroupingMove(int oldGroup, int index, int newGroup, MultiComponentVectorEncoding multiComponentVector)
      : base(multiComponentVector) {
        OldGroup = oldGroup;
        Index = index;
        NewGroup = newGroup;
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new SingleGroupingMove(this, cloner);
    }

    public override IPackingSolutionEncoding GetSolutionAfterMove() {
      return GetVectorAfterMove(MultiComponentVector, OldGroup, Index, NewGroup);
    }
    public static MultiComponentVectorEncoding GetVectorAfterMove(MultiComponentVectorEncoding originalSolution, int oldBin, int index, int newBin) {
      var solution = originalSolution.Clone (new Cloner()) as MultiComponentVectorEncoding;
      var itemgene = solution.PackingInformations[oldBin][index];
      solution.PackingInformations[oldBin].Remove(itemgene);
      solution.PackingInformations[newBin].Add(itemgene);
      return solution;
    }

    public override System.Type GetMoveAttributeType() {
      return typeof(SingleGroupingMoveAttribute);
    }

    public override string ToString() {
      return "GM(og"+OldGroup+",i="+Index+",ng="+NewGroup+")";
    }

    public override MultiComponentVectorMoveAttribute GetAttribute(double quality) {
      return new SingleGroupingMoveAttribute(OldGroup, Index,
        MultiComponentVector.PackingInformations[OldGroup][Index].ItemID, NewGroup, quality);
    }

    public override bool BelongsToAttribute(MultiComponentVectorMoveAttribute attribute, bool hardCriterion) {
      SingleGroupingMoveAttribute actualAttribute = attribute as SingleGroupingMoveAttribute;
      if (actualAttribute != null) {
        if (hardCriterion) {
          if (MultiComponentVector.PackingInformations[OldGroup][Index].ItemID == actualAttribute.ItemID
                || NewGroup == actualAttribute.NewGroup)
            return true;
        } else {
          if (MultiComponentVector.PackingInformations[OldGroup][Index].ItemID == actualAttribute.ItemID
                && NewGroup == actualAttribute.NewGroup)
            return true;
        }
      }
      return false;
    }
  }
}